@@ -84,11 +84,11 @@ class SNPs:
             name / path of resources directory
         deduplicate : bool
             deduplicate RSIDs and make SNPs available as `SNPs.duplicate`
-        deduplicate_XY_chrom : bool
-            deduplicate alleles in the non-PAR regions of X and Y for males; see
-            `SNPs.discrepant_XY`
         deduplicate_MT_chrom : bool
             deduplicate alleles on MT; see `SNPs.heterozygous_MT`
+        deduplicate_XY_chrom : bool or str
+            deduplicate alleles in the non-PAR regions of X and Y for males; see `SNPs.discrepant_XY`
+            if a `str` then this is the sex determination method to use X Y or XY
         parallelize : bool
             utilize multiprocessing to speedup calculations
         processes : int
@@ -131,6 +131,8 @@ class SNPs:
                 d["source"].split(", ") if ", " in d["source"] else [d["source"]]
             )
             self._phased = d["phased"]
+            self._build = d["build"]
+            self._build_detected = True if d["build"] else False
 
             if not self._snps.empty:
                 self.sort()
@@ -138,13 +140,11 @@ class SNPs:
                 if deduplicate:
                     self._deduplicate_rsids()
 
-                # prefer to use SNP positions to detect build
-                self._build = self.detect_build()
-                self._build_detected = True if self._build else False
-
+                # use build detected from `read` method or comments, if any
+                # otherwise use SNP positions to detect build
                 if not self._build_detected:
-                    # use build detected from `read` method or comments, if any
-                    self._build = d["build"]
+                    self._build = self.detect_build()
+                    self._build_detected = True if self._build else False
 
                     if not self._build:
                         self._build = 37  # assume Build 37 / GRCh37 if not detected
@@ -156,7 +156,9 @@ class SNPs:
                     self.sort()
 
                 if deduplicate_XY_chrom:
-                    if self.determine_sex() == "Male":
+                    if (
+                        deduplicate_XY_chrom is True and self.determine_sex() == "Male"
+                    ) or self.determine_sex(chrom=deduplicate_XY_chrom) == "Male":
                         self._deduplicate_XY_chrom()
 
                 if deduplicate_MT_chrom:
@@ -164,8 +166,14 @@ class SNPs:
             else:
                 logger.warning("no SNPs loaded...")
 
+    def __len__(self):
+        return self.count
+
     def __repr__(self):
-        return f"SNPs({self._file[0:50]!r})"
+        if isinstance(self._file, str):
+            return f"SNPs('{os.path.basename(self._file)}')"
+        else:
+            return "SNPs(<bytes>)"
 
     @property
     def source(self):
@@ -991,8 +999,9 @@ class SNPs:
 
     def sort(self):
         """ Sort SNPs based on ordered chromosome list and position. """
-
-        sorted_list = sorted(self._snps["chrom"].unique(), key=self._natural_sort_key)
+        sorted_list = sorted(
+            (str(x) for x in self._snps["chrom"].unique()), key=self._natural_sort_key
+        )
 
         # move PAR and MT to the end of the dataframe
         if "PAR" in sorted_list:
@@ -1159,58 +1168,68 @@ class SNPs:
         pos_end = int(temp["pos"].describe()["max"])
 
         for mapping in mappings["mappings"]:
-            # skip if mapping is outside of range of SNP positions
-            if (
-                mapping["original"]["end"] < pos_start
-                or mapping["original"]["start"] > pos_end
-            ):
-                continue
 
-            orig_range_len = mapping["original"]["end"] - mapping["original"]["start"]
-            mapped_range_len = mapping["mapped"]["end"] - mapping["mapped"]["start"]
+            orig_start = mapping["original"]["start"]
+            orig_end = mapping["original"]["end"]
+            mapped_start = mapping["mapped"]["start"]
+            mapped_end = mapping["mapped"]["end"]
 
             orig_region = mapping["original"]["seq_region_name"]
             mapped_region = mapping["mapped"]["seq_region_name"]
 
-            if orig_region != mapped_region:
-                logger.warning("discrepant chroms")
-                continue
-
-            if orig_range_len != mapped_range_len:
-                logger.warning(
-                    "discrepant coords"
-                )  # observed when mapping NCBI36 -> GRCh38
+            # skip if mapping is outside of range of SNP positions
+            if orig_end < pos_start or orig_start > pos_end:
                 continue
 
             # find the SNPs that are being remapped for this mapping
             snp_indices = temp.loc[
                 ~temp["remapped"]
-                & (temp["pos"] >= mapping["original"]["start"])
-                & (temp["pos"] <= mapping["original"]["end"])
+                & (temp["pos"] >= orig_start)
+                & (temp["pos"] <= orig_end)
             ].index
 
-            if len(snp_indices) > 0:
-                # remap the SNPs
-                if mapping["mapped"]["strand"] == -1:
-                    # flip and (optionally) complement since we're mapping to minus strand
-                    diff_from_start = (
-                        temp.loc[snp_indices, "pos"] - mapping["original"]["start"]
-                    )
-                    temp.loc[snp_indices, "pos"] = (
-                        mapping["mapped"]["end"] - diff_from_start
-                    )
+            # if there are no snp here, skip
+            if not len(snp_indices):
+                continue
 
-                    if complement_bases:
-                        temp.loc[snp_indices, "genotype"] = temp.loc[
-                            snp_indices, "genotype"
-                        ].apply(self._complement_bases)
-                else:
-                    # mapping is on same (plus) strand, so just remap based on offset
-                    offset = mapping["mapped"]["start"] - mapping["original"]["start"]
-                    temp.loc[snp_indices, "pos"] = temp["pos"] + offset
+            orig_range_len = orig_end - orig_start
+            mapped_range_len = mapped_end - mapped_start
 
-                # mark these SNPs as remapped
-                temp.loc[snp_indices, "remapped"] = True
+            # if this would change chromosome, skip
+            # TODO allow within normal chromosomes
+            # TODO flatten patches
+            if orig_region != mapped_region:
+                logger.warning(
+                    f"discrepant chroms for {len(snp_indices)} SNPs from {orig_region} to {mapped_region}"
+                )
+                continue
+
+            # if there is any stretching or squashing of the region
+            # observed when mapping NCBI36 -> GRCh38
+            # TODO disallow skipping a version when remapping
+            if orig_range_len != mapped_range_len:
+                logger.warning(
+                    f"discrepant coords for {len(snp_indices)} SNPs from {orig_region}:{orig_start}-{orig_end} to {mapped_region}:{mapped_start}-{mapped_end}"
+                )
+                continue
+
+            # remap the SNPs
+            if mapping["mapped"]["strand"] == -1:
+                # flip and (optionally) complement since we're mapping to minus strand
+                diff_from_start = temp.loc[snp_indices, "pos"] - orig_start
+                temp.loc[snp_indices, "pos"] = mapped_end - diff_from_start
+
+                if complement_bases:
+                    temp.loc[snp_indices, "genotype"] = temp.loc[
+                        snp_indices, "genotype"
+                    ].apply(self._complement_bases)
+            else:
+                # mapping is on same (plus) strand, so just remap based on offset
+                offset = mapped_start - orig_start
+                temp.loc[snp_indices, "pos"] = temp["pos"] + offset
+
+            # mark these SNPs as remapped
+            temp.loc[snp_indices, "remapped"] = True
 
         return temp
 
