@@ -362,6 +362,12 @@ class TestSnps(BaseSNPsTestCase):
         s.save(out)
         self.assertTrue(out.read().startswith("# Generated by snps"))
 
+    def test_save_buffer_binary(self):
+        s = SNPs("tests/input/generic.csv")
+        out = io.BytesIO()
+        s.save(out)
+        self.assertTrue(out.read().startswith(b"# Generated by snps"))
+
     def test_save_no_snps(self):
         s = SNPs()
         self.assertFalse(s.save())
